# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/nc-to-thredds/source/golf_parameters.py $
# Last committed:     $Revision: 407352 $; $Author: vvuik $; $Date: 2019-11-05 17:32:50 +0100 (di, 05 nov 2019) $
#
# -*- coding: utf-8 -*-
"""
Created on Thu Jun 27 13:43:43 2019

@author: hove
"""

import numpy as np
import pandas as pd
import datetime

from spectrum1D import spectrum1D
from calcmoments import calcmoments

def golf_parameters(df, startTime, endTime, instrument):
    
    tstep = 15 # time step in minutes
    
    # settings
    if instrument=='AQD':
        bLen = 14 # burst length in minutes
    else:
        bLen = tstep # burst length in minutes
    
    freq = 4    # sampling frequency in Hz
    maxE = 0.01 # maximum fraction of empty values in burst
    fmin = 0.10 # minimum frequency for calculating spectral parameters (100-1500 mHz)
    fmax = 1.50 # maximum frequency for calculating spectral parameters (100-1500 mHz)
    
    # assume uniform spacing between samples
    bNum = bLen*60*freq # expected number of values in burst
    eMax = np.ceil(maxE*bNum) # maximum number of empty  values in burst
    
    # start indices
    i0  = 0
    t0  = startTime
    b   = 1 # burst number
    
    # dataframe to store results
    li = pd.DataFrame(columns = ['hmean','Hm0','Tp','Tm01','Tm02','Tmm10'], dtype='float')
    spec = pd.DataFrame(columns = ['vy'], dtype='object')
    fx = None # initialize
        
    while 1:
    
        # update time and index
        t1 = t0+datetime.timedelta(minutes=tstep)
        i1 = df.index.get_loc(t1,method='nearest')
        
        if t1 > endTime:
            break
        
        # select burst
        hsel = df.iloc[i0:i1]['H.w.0'].astype('float')
        hsel = hsel.replace(-999,np.NaN)
        
        # reindex dataframe to insert np.NaN for missing timesteps
        if instrument=='AQD':
            # hier gaan we 14 minuten uithalen vanaf de eerste not nan 
            if any(hsel.notnull()):
                st = hsel[hsel.notnull()].index[0]
            else:
                st = t0
            date_range = pd.date_range(st, st+datetime.timedelta(minutes=bLen), freq='250ms', closed='left')
        else:
            date_range = pd.date_range(t0, t1, freq='250ms', tz='Etc/GMT', closed='left')
        hsel = hsel.reindex(date_range)
        
        # if burst has sufficient finite values (fraction>1-maxE)
        if hsel.isna().sum()<=eMax:
            
            # interpolate missing values if less than eMax
            hmean = np.nanmean(hsel)
            hsel = hsel.interpolate(method = 'time', limit_direction = 'both').ffill().bfill()

            # perform spectral analysis
            tvec = np.arange(0, bLen*60, 1/freq) # assume uniform spacing
            fx,vy = spectrum1D(tvec, hsel)
            
            spec.loc[t0, 'vy'] = vy
            
            # calculating spectral parameters
            ef    = np.where(np.logical_and(fx>=fmin,fx<=fmax))
            fsel  = fx[ef]
            vsel  = vy[ef]
            em    = np.argmax(vsel) 
            fp    = fsel[em] # peak frequency								
            Tp    = 1/fp # peak period
            mom   = calcmoments(fx,vy,fmin=fmin,fmax=fmax)
            Hm0   = float(4*np.sqrt(mom['m0'])) # Hm0 = 4*m0^0.5
            Tm01  = float(mom['m0']/mom['m1']) # Tm01 = m0/m1
            Tm02  = float(np.sqrt(mom['m0']/mom['m2'])) # Tm02 = (m0/m2)^0.5
            Tmm10 = float(mom['mm1']/mom['m0']) # Tm-1,0 = (m-1/m0)    
            li.loc[t0, :] = [hmean, Hm0, Tp, Tm01, Tm02, Tmm10]
            
        else:
            print('---')
            print('warning: no spectral analysis at '+ t0.strftime("%Y%m%d_%H%M%S"))
            print('samples in burst: ' + str(np.size(hsel)))
            print('NaNs in burst: ' + str(hsel.isna().sum()))
            print('elements df: ' + str(i0)+':'+str(i1))
            print('---')
        
        # shift to next burst
        b = b+1 
        i0 = i1
        t0 = t1
        
    return li, spec, fx